//
//  ScannerOverlay.swift
//  Alamofire
//
//  Created by Amit satyapanthi on 11/03/19.
//

import Foundation

class ScannerOverlay: UIView {
    
    //var scanLineRect = CGRect.zero
    var line:UIView!
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        line = UIView()
        line.backgroundColor = UIColor.red
        line.translatesAutoresizingMaskIntoConstraints = false
        addSubview(line)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func draw(_ rect: CGRect) {
        var context = UIGraphicsGetCurrentContext()
        
        var overlayColor = UIColor(red: 0.0, green: 0.0, blue: 0.0, alpha: 0.55)
        var scanLineColor = UIColor.red
        
        context?.setFillColor(overlayColor.cgColor)
        context?.fill(bounds)
        
        // make a hole for the scanner
        var holeRect: CGRect = scanRect()
        var holeRectIntersection: CGRect = holeRect.intersection(rect)
        UIColor.clear.setFill()
        UIRectFill(holeRectIntersection)
        
        // draw a horizontal line over the middle
        
        // draw a horizontal line over the middle
        var lineRect: CGRect = scanLineRect()
        line.frame = lineRect
        
        // drw the green corners
        var cornerSize: CGFloat = 30
        var path = UIBezierPath()
        //top left corner
        path.move(to: CGPoint(x: holeRect.origin.x, y: holeRect.origin.y + cornerSize))
        path.addLine(to: CGPoint(x: holeRect.origin.x, y: holeRect.origin.y))
        path.addLine(to: CGPoint(x: holeRect.origin.x + cornerSize, y: holeRect.origin.y))

        //top right corner
        
        //top right corner
        var rightHoleX: CGFloat = holeRect.origin.x + holeRect.size.width
        path.move(to: CGPoint(x: rightHoleX - cornerSize, y: holeRect.origin.y))
        path.addLine(to: CGPoint(x: rightHoleX, y: holeRect.origin.y))
        path.addLine(to: CGPoint(x: rightHoleX, y: holeRect.origin.y + cornerSize))
        
        // bottom right corner
        var bottomHoleY: CGFloat = holeRect.origin.y + holeRect.size.height
        path.move(to: CGPoint(x: rightHoleX, y: bottomHoleY - cornerSize))
        path.addLine(to: CGPoint(x: rightHoleX, y: bottomHoleY))
        path.addLine(to: CGPoint(x: rightHoleX - cornerSize, y: bottomHoleY))
        
        // bottom left corner
        path.move(to: CGPoint(x: holeRect.origin.x + cornerSize, y: bottomHoleY))
        path.addLine(to: CGPoint(x: holeRect.origin.x, y: bottomHoleY))
        path.addLine(to: CGPoint(x: holeRect.origin.x, y: bottomHoleY - cornerSize))
        
        path.lineWidth = 2
        UIColor.green.setStroke()
        path.stroke()
    }
    
    func startAnimating() {
        let flash = CABasicAnimation(keyPath: "opacity")
        flash.fromValue = NSNumber(value: 0.0)
        flash.toValue = NSNumber(value: 1.0)
        flash.duration = 0.25
        flash.autoreverses = true
        flash.repeatCount = .infinity
        line.layer.add(flash, forKey: "flashAnimation")
    }
    
    func stopAnimating() {
        layer.removeAnimation(forKey: "flashAnimation")
    }
    
    func scanRect() -> CGRect {
        let rect: CGRect = frame
        let heightMultiplier: CGFloat = 3.0 / 4.0 // 4:3 aspect ratio
        let scanRectWidth: CGFloat = rect.size.width * 0.8
        let scanRectHeight: CGFloat = scanRectWidth * heightMultiplier
        let scanRectOriginX: CGFloat = (rect.size.width / 2) - (scanRectWidth / 2)
        let scanRectOriginY: CGFloat = (rect.size.height / 2) - (scanRectHeight / 2)
        return CGRect(x: scanRectOriginX, y: scanRectOriginY, width: scanRectWidth, height: scanRectHeight)
    }
    
    func scanLineRect() -> CGRect {
        let scanRect: CGRect = self.scanRect()
        let rect: CGRect = frame
        return CGRect(x: scanRect.origin.x, y: rect.size.height / 2, width: scanRect.size.width, height: 1)
    }
}
